import tm_skeleton
    
class DTM(tm_skeleton.DTM):    
    
    def start(self, input_word):
        self.state: TState = self.code.initial
        self.pos: int  = 1
        self.tape: list[str]  = ["^"] + [i for i in input_word]

    def next(self, print=False):        
        transition = self.code.transition[self.state][self.tape[self.pos]]
        self.state = transition[0]
        self.tape[self.pos] = transition[1]
        self.pos += transition[2]
        
        if self.pos>=len(self.tape):
            self.tape.append("_")
        elif self.pos < 0:
            raise Exception("End of tape.")

        if print:
            self.print()
            
        if self.state in self.code.reject:
            raise Exception(f"Rejected with state {self.state}.")
        else:
            return self.state in self.code.accept
            
        
    def exec(self, step=-1, print=False):
        if print:
            self.print()
        i=0
        while(not(self.next(print=print)) and (i<step or step==-1)):
            i += 1

    def print(self):
        tapestr_pref = f"{'|'.join(self.tape[0:self.pos+1])}"
        tapestr = f"|{'|'.join(self.tape)}|"
        print(tapestr)
        print(f"{' '*(len(tapestr_pref))}↑  {self.state}")

if __name__ == "__main__":
    import json
    from sys import argv

    path = "asbs.json"
    input_word = "aab"

    if len(argv) > 1:
        path = argv[1]
        input_word = argv[2]
    # Usage example
    ## Load the code from asbs.json; states are represented by strings 
    with open(path) as f:
        code = tm_skeleton.DTMCode(**json.load(f))
    
    ## Load the code into the machine
    machine_asbs = DTM(code)
    
    ## Initialize the tape
    machine_asbs.start(input_word)
    
    ## Print the initial configuration
    ## |^|a|b|b|
    ##    ↑  start
    machine_asbs.print()
    
    ## Execute the machine
    machine_asbs.exec(print=True)
    
    ## Should produce:
    
    # |^|a|b|b|
    #    ↑  start
    # |^|a|b|b|
    #      ↑  start
    # |^|a|b|b|
    #        ↑  readb
    # |^|a|b|b|_|
    #          ↑  readb
    # |^|a|b|b|_|
    #          ↑  accept
